module subrotinas
  contains

  subroutine mapa(y, x)
    use constantes
    implicit none
    real(8), intent(inout) :: x, y

    y = y - b * sin(p2 * x) / (1.d0 - mu * cos(p2 * x))
    x = x + a * (1 - y**2)

    if (x >= 1.0) then
      x = x - int(x)
    else if (x < 0.0) then
      x = x - int(x) + 1.0
    end if

    return
  end subroutine mapa

  subroutine mapa_unbound(y, x)
    use constantes
    implicit none
    real(8), intent(inout) :: x, y

    y = y - b * sin(p2 * x) / (1.d0 - mu * cos(p2 * x))
    x = x + a * (1 - y**2)

    return
  end subroutine mapa_unbound

  subroutine jacobian_matrix(dm, x, y)
    use constantes
    implicit none
    real(8), intent(in) :: x, y
    real(8), dimension(2,2), intent(out) :: dm

    dm(1,1) = 1.d0
    dm(1,2) = -b * p2 * cos(p2 * x) / (1.0 - mu * cos(p2 * x)) + &
              b * p2 * mu * sin(p2 * x)**2 / (1.0 - mu * cos(p2 * x))**2
    dm(2,1) = -2.0 * a * (y - b * sin(p2 * x) / (1.0 - mu * cos(p2 * x)))
    dm(2,2) = 1.0 - 2.0 * a * (y - b * sin(p2 * x) / (1.0 - mu * cos(p2 * x))) * &
              (-b * p2 * cos(p2 * x) / (1.0 - mu * cos(p2 * x)) + &
              b * p2 * mu * sin(p2 * x)**2 / (1.0 - mu * cos(p2 * x))**2)

    return
  end subroutine jacobian_matrix

end module subrotinas
