module subrotinas
  contains

  subroutine mapa(y, x, a)
    use constantes
    implicit none
    real(8), intent(inout) :: x, y
    real(8), intent(in) :: a

    y = y - b * sin(p2 * x) / (1.d0 - mu * cos(p2 * x))
    x = x + a * (1 - y**2)

    if (x >= 1.0) then
      x = x - int(x)
    else if (x < 0.0) then
      x = x - int(x) + 1.0
    end if

    return
  end subroutine mapa

  subroutine jacobian_matrix(dm, x, y)
    use constantes
    implicit none
    real(8), intent(in) :: x, y
    real(8), dimension(2,2), intent(out) :: dm

    dm(1,1) = 1.d0
    dm(1,2) = -b * p2 * cos(p2 * x) / (1.0 - mu * cos(p2 * x)) + &
              b * p2 * mu * sin(p2 * x)**2 / (1.0 - mu * cos(p2 * x))**2
    dm(2,1) = -2.0 * a * (y - b * sin(p2 * x) / (1.0 - mu * cos(p2 * x)))
    dm(2,2) = 1.0 - 2.0 * a * (y - b * sin(p2 * x) / (1.0 - mu * cos(p2 * x))) * &
              (-b * p2 * cos(p2 * x) / (1.0 - mu * cos(p2 * x)) + &
              b * p2 * mu * sin(p2 * x)**2 / (1.0 - mu * cos(p2 * x))**2)

    return
  end subroutine jacobian_matrix

  subroutine lyap(ortogonal, triangular, x, y)
    use constantes
    implicit none
    real(8), intent(in) :: x, y
    real(8), dimension(2,2), intent(inout) :: ortogonal
    real(8), dimension(2,2), intent(out) :: triangular
    real(8), dimension(2,2) :: dmLinha, dm
    real(8) :: coseno, seno, h

    call jacobian_matrix(dm, x, y)

    dmLinha = matmul(dm, ortogonal)

    h = sqrt(dmLinha(2,1)**2 + dmLinha(1,1)**2)
    coseno = dmLinha(1,1) / h
    seno = dmLinha(2,1) / h

    triangular(1,1) = dmLinha(1,1) * coseno + dmLinha(2,1) * seno
    triangular(2,2) = -dmLinha(1,2) * seno + dmLinha(2,2) * coseno

    ortogonal(1,1) = coseno
    ortogonal(1,2) = -seno
    ortogonal(2,1) = seno
    ortogonal(2,2) = coseno

    return
  end subroutine lyap

end module subrotinas
