module subrotinas
contains

  subroutine mapa(y, x)
    use constantes
    implicit none
    real(8), intent(inout) :: x, y

    y = y - b * sin(p2 * x) / (1.d0 - mu * cos(p2 * x))
    x = x + a * (1 - y**2)

    if (x >= 1.0) then
      x = x - int(x)
    else if (x < 0.0) then
      x = x - int(x) + 1.0
    end if

    return
  end subroutine mapa

subroutine mapa_inv(y, x)
  use constantes
  implicit none
  real(8) x, y

  x = x - a * (1 - y**2)
  y = y + b * sin(p2 * x)/ (1.d0 - mu * cos(p2 * x))

  if (x >= 1.0) then
    x = x - int(x)
  else if (x < 0.0) then
    x = x - int(x) + 1.0
  end if

return
end subroutine mapa_inv

subroutine distancia (y_1, x_1, y_0, x_0, delta_x, delta_y)
  use constantes
  implicit none
  real(8), intent(in) :: x_0, y_0, x_1, y_1
  real(8), intent(out) :: delta_x, delta_y
  
  delta_x = abs(x_1-x_0)
  if (delta_x > 0.5d0) delta_x = 1.d0 - delta_x
  delta_y = abs(y_1 - y_0)
  
  return
end subroutine distancia

subroutine laplacian_filter(input_array, output_array, nx, ny)
  implicit none
  real(8), dimension(nx,ny), intent(in) :: input_array
  real(8), dimension(nx,ny), intent(out) :: output_array
  integer, intent(in) :: nx, ny
  integer :: i, j
  do i=2,nx-1
    do j=2,ny-1
      output_array(i,j) = -4.d0*input_array(i,j) + &
              input_array(i-1,j) + input_array(i+1,j) + &
              input_array(i,j-1) + input_array(i,j+1)
    end do
  end do
  ! termos de fronteira
  do j=2,ny-1
    output_array(1,j) = - input_array(1,j) - 2.d0*input_array(2,j) + &
            input_array(3,j) + input_array(1,j+1) + input_array(1,j-1)
    output_array(nx,j) = - input_array(nx,j) - 2.d0*input_array(nx-1,j) + &
            input_array(nx-2,j) + input_array(nx,j+1) + input_array(nx,j-1)              
  end do
  do i=2,nx-1
    output_array(i,1) = - input_array(i,1) - 2.d0*input_array(i,2) + &
            input_array(i,3) + input_array(i+1,1) + input_array(i-1,1)
    output_array(i,ny) = - input_array(i,ny) - 2.d0*input_array(i,ny-1) + &
            input_array(i,ny-2) + input_array(i+1,ny) + input_array(i-1,ny)        
  end do
  output_array(1,1) = 2.d0*input_array(1,1) - 2.d0*input_array(2,1) + &
          input_array(3,1) - 2.d0*input_array(1,2) + input_array(1,3)
  output_array(1,ny) = 2.d0*input_array(1,ny) - 2.d0*input_array(2,ny) + &
          input_array(3,ny) - 2.d0*input_array(1,ny-1) + input_array(1,ny-2)
  output_array(nx,1) = 2.d0*input_array(nx,1) - 2.d0*input_array(nx-1,1) + &
          input_array(nx-2,1) - 2.d0*input_array(nx,2) + input_array(nx,3)                  
  output_array(nx,ny) = 2.d0*input_array(nx,ny) - 2.d0*input_array(nx-1,ny) + &
          input_array(nx-2,1) - 2.d0*input_array(nx,ny-1) + input_array(nx,ny-2)  
end subroutine laplacian_filter

end module

